@@ -27,6 +27,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include <string>
 #include <vector>
 
+class Collision;
 class DrawList;
 class Flotsam;
 class Minable;
@@ -58,9 +59,11 @@ public:
 	void Step(std::vector<Visual> &visuals, std::list<std::shared_ptr<Flotsam>> &flotsam, int step);
 	// Draw the asteroid field, with the field of view centered on the given point.
 	void Draw(DrawList &draw, const Point &center, double zoom) const;
-	// Check if the given projectile has hit any of the asteroids, using the information
-	// in the collision sets. If a collision occurs, returns a pointer to the hit body.
-	Body *Collide(const Projectile &projectile, double *closestHit);
+
+	// Check if the given projectile collides with any asteroids. This excludes minables.
+	const std::vector<Collision> &CollideAsteroids(const Projectile &projectile) const;
+	// Check if the given projectile collides with any minables.
+	const std::vector<Collision> &CollideMinables(const Projectile &projectile) const;
 
 	// Get the list of minable asteroids.
 	const std::list<std::shared_ptr<Minable>> &Minables() const;
@@ -88,6 +91,9 @@ private:
 
 	CollisionSet asteroidCollisions;
 	CollisionSet minableCollisions;
+
+	// Vector for returning the result of CollideAsteroids.
+	mutable std::vector<Collision> result;
 };
 
 
