@@ -160,7 +160,9 @@ namespace {
 		{"shield generation multiplier", -1.},
 		{"shield energy multiplier", -1.},
 		{"shield fuel multiplier", -1.},
-		{"shield heat multiplier", -1.}
+		{"shield heat multiplier", -1.},
+		{"acceleration multiplier", -1.},
+		{"turn multiplier", -1.}
 	};
 
 	void AddFlareSprites(vector<pair<Body, int>> &thisFlares, const pair<Body, int> &it, int count)
@@ -275,20 +277,15 @@ void Outfit::Load(const DataNode &node)
 			mass = child.Value(1);
 		else if(child.Token(0) == "licenses" && (child.HasChildren() || child.Size() >= 2))
 		{
-			auto isNewLicense = [](const vector<string> &c, const string &val) noexcept -> bool {
-				return find(c.begin(), c.end(), val) == c.end();
-			};
 			// Add any new licenses that were specified "inline".
 			if(child.Size() >= 2)
 			{
 				for(auto it = ++begin(child.Tokens()); it != end(child.Tokens()); ++it)
-					if(isNewLicense(licenses, *it))
-						licenses.push_back(*it);
+					AddLicense(*it);
 			}
 			// Add any new licenses that were specified as an indented list.
 			for(const DataNode &grand : child)
-				if(isNewLicense(licenses, grand.Token(0)))
-					licenses.push_back(grand.Token(0));
+				AddLicense(grand.Token(0));
 		}
 		else if(child.Token(0) == "jump range" && child.Size() >= 2)
 		{
@@ -325,7 +322,8 @@ void Outfit::Load(const DataNode &node)
 		GameData::AddJumpRange(attributes.Get("jump range"));
 
 	// Legacy support for turrets that don't specify a turn rate:
-	if(IsWeapon() && attributes.Get("turret mounts") && !TurretTurn() && !AntiMissile())
+	if(IsWeapon() && attributes.Get("turret mounts") && !TurretTurn()
+		&& !AntiMissile() && !TractorBeam())
 	{
 		SetTurretTurn(4.);
 		node.PrintTrace("Warning: Deprecated use of a turret without specified \"turret turn\":");
@@ -542,6 +540,14 @@ void Outfit::Add(const Outfit &other, int count)
 
 
 
+void Outfit::AddLicenses(const Outfit &other)
+{
+	for(const auto &license : other.licenses)
+		AddLicense(license);
+}
+
+
+
 // Modify this outfit's attributes.
 void Outfit::Set(const char *attribute, double value)
 {
@@ -656,3 +662,13 @@ const Sprite *Outfit::FlotsamSprite() const
 {
 	return flotsamSprite;
 }
+
+
+
+// Add the license with the given name to the licenses required by this outfit, if it is not already present.
+void Outfit::AddLicense(const string &name)
+{
+	const auto it = find(licenses.begin(), licenses.end(), name);
+	if(it == licenses.end())
+		licenses.push_back(name);
+}
